@@ -1289,6 +1289,8 @@ impl<E: ExactBytesEncode> ExactBytesEncode for Slice<E> {
 }
 
 /// Combinator for representing encoders that cannot accept any more items.
+///
+/// This is created by calling `EncodeExt::last`.
 #[derive(Debug, Default)]
 pub struct Last<E> {
     inner: E,
@@ -1338,12 +1340,129 @@ impl<E: ExactBytesEncode> ExactBytesEncode for Last<E> {
     }
 }
 
+/// Combinator for representing encoders that accepts only one additional item.
+///
+/// This is created by calling `EncodeExt::last_item`.
+#[derive(Debug, Default)]
+pub struct LastItem<E: Encode> {
+    inner: E,
+    item: Option<E::Item>,
+}
+impl<E: Encode> LastItem<E> {
+    /// Returns a reference to the inner encoder.
+    pub fn inner_ref(&self) -> &E {
+        &self.inner
+    }
+
+    /// Returns a mutable reference to the inner encoder.
+    pub fn inner_mut(&mut self) -> &mut E {
+        &mut self.inner
+    }
+
+    /// Takes ownership of this instance and returns the inner encoder.
+    pub fn into_inner(self) -> E {
+        self.inner
+    }
+
+    pub(crate) fn new(inner: E, item: E::Item) -> Self {
+        LastItem {
+            inner,
+            item: Some(item),
+        }
+    }
+}
+impl<E: Encode> Encode for LastItem<E> {
+    type Item = Never;
+
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        if self.inner.is_idle() {
+            if let Some(item) = self.item.take() {
+                track!(self.inner.start_encoding(item))?;
+            }
+        }
+        track!(self.inner.encode(buf, eos))
+    }
+
+    fn start_encoding(&mut self, _item: Self::Item) -> Result<()> {
+        unreachable!()
+    }
+
+    fn is_idle(&self) -> bool {
+        self.item.is_none() && self.inner.is_idle()
+    }
+
+    fn requiring_bytes(&self) -> ByteCount {
+        if self.item.is_some() {
+            ByteCount::Unknown
+        } else {
+            self.inner.requiring_bytes()
+        }
+    }
+}
+
+/// Combinator for allowing EOS reached before decoding starts.
+#[derive(Debug, Default)]
+pub struct MaybeEos<D> {
+    inner: D,
+    started: bool,
+}
+impl<D> MaybeEos<D> {
+    /// Makes a new `MaybeEos` instance.
+    pub fn new(inner: D) -> Self {
+        MaybeEos {
+            inner,
+            started: false,
+        }
+    }
+
+    /// Returns a reference to the inner decoder.
+    pub fn inner_ref(&self) -> &D {
+        &self.inner
+    }
+
+    /// Returns a mutable reference to the inner decoder.
+    pub fn inner_mut(&mut self) -> &mut D {
+        &mut self.inner
+    }
+
+    /// Takes ownership of this instance and returns the inner decoder.
+    pub fn into_inner(self) -> D {
+        self.inner
+    }
+}
+impl<D: Decode> Decode for MaybeEos<D> {
+    type Item = D::Item;
+
+    fn decode(&mut self, buf: &[u8], mut eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        if !self.started && eos.is_reached() {
+            eos = Eos::new(false);
+        }
+        let (size, item) = track!(self.inner.decode(buf, eos))?;
+        if let Some(item) = item {
+            self.started = false;
+            Ok((size, Some(item)))
+        } else {
+            self.started |= size > 0;
+            Ok((size, None))
+        }
+    }
+
+    fn has_terminated(&self) -> bool {
+        self.inner.has_terminated()
+    }
+
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
+    }
+}
+
 #[cfg(test)]
 mod test {
     use {Decode, DecodeExt, Encode, EncodeExt, Eos, ErrorKind};
     use bytes::{Utf8Decoder, Utf8Encoder};
-    use fixnum::{U8Decoder, U8Encoder};
+    use fixnum::{U16beDecoder, U8Decoder, U8Encoder};
     use io::{IoDecodeExt, IoEncodeExt};
+    use super::*;
 
     #[test]
     fn collect_works() {
@@ -1492,4 +1611,17 @@ mod test {
         let (_, item) = track_try_unwrap!(decoder.decode(b"\x03foo", Eos::new(false)));
         assert_eq!(item, Some("foo".to_owned()));
     }
+
+    #[test]
+    fn maybe_eos_works() {
+        let mut decoder = U16beDecoder::new();
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
+
+        let mut decoder = MaybeEos::new(U16beDecoder::new());
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_ok());
+
+        let mut decoder = MaybeEos::new(U16beDecoder::new());
+        assert!(decoder.decode(&[1][..], Eos::new(false)).is_ok());
+        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
+    }
 }
